function [CTT,I,map,DF,nbr_of_routers] = graph2inputs(g,DF) 
% GRAPH2INPUTS converts object graph to the matrixes.
%
% Parameters:
%   g   object graph (output from graphedit tool)
%   DF  set of required data flows in WSN. Each data flow is desribed by
%   the structure:
%       DF.sources          set of sources (router or end-node) of the data
%       flow
%       DF.sink             a sink (router or end-node) of the flow
%       DF.deadline         [sec] deadline to message delivery from any
%       source to the sink
%       DF.sample_period    [sec] sample period of sensing data
%       DF.sammple_size     [bits] sample size of sensing data
%       DF.ack              [true/false] retransmission (packet with
%       acknowledgment)
%
%   CTT     description of cluster-tree topology CTT (logical topology) -
%   parent-child relation. Adjacency matrix (directed graph): CTTij=1 if
%   node j is parent of node i. CTTij=0 otherwise. indexes: [routers,end_nodes]
%
%   I       interference matrix.Iij=1 if a direct or indirect interference
%   is between clusters (routers) i and j (i.e. clusters i and j cannot be
%   active at the same time). Iii = 2;
%
%   map     mapping nodes' index between object graph and matrix. 
%           [graph index (T_{i}), node address, x, y ] 
%
%   DF   data flows with mapped nodes' index
%   nbr_of_routers  number of routers
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

router = struct('color',[1 1 0],'curvature',[1 1]);
end_node = struct('color',[1 0 0],'curvature',[0]);

radio_range = g.UserParam.graphedit.plugin.noderange; %radio radius

nbr_of_nodes = length(g.ndl);

end_nodes = [];
nbr_of_routers = 0;
map = [];
for i=1:nbr_of_nodes
    if g.N(i).GraphicParam(1).facecolor == router.color
        nbr_of_routers = nbr_of_routers +1;
        x = g.N(i).GraphicParam(1).x + g.N(i).GraphicParam(1).width/2;
        y = g.N(i).GraphicParam(1).y + g.N(i).GraphicParam(1).height/2;
        map=[map [i,cell2mat(g.N(i).Userparam),x,y]'];  %map [graph index (T_{i}), node address, x, y ]'
    elseif g.N(i).GraphicParam(1).facecolor == end_node.color
        end_nodes = [end_nodes i];
    end
end

for i=1:nbr_of_nodes-nbr_of_routers
    index = end_nodes(i);
    x = g.N(index).GraphicParam(1).x + g.N(index).GraphicParam(1).width/2;
    y = g.N(index).GraphicParam(1).y + g.N(index).GraphicParam(1).height/2;
    map=[map [index,cell2mat(g.N(index).Userparam),x,y]'];  %map [graph index (T_{i}), node address, x, y ]'
end

CTT = g.adj; %adjacency matrix
CTT_temp = zeros(size(CTT));
for i=1:nbr_of_nodes
    CTT_temp(:,i) = CTT(:,map(1,i));
end

for i=1:nbr_of_nodes
    CTT(i,:) = CTT_temp(map(1,i),:);
end

I = zeros(nbr_of_routers);

for i=1:nbr_of_routers
    A_x = map(3,i);
    A_y = map(4,i);
    
    %direct interfaces
    for j=1:nbr_of_routers
        if j~=i
            B_x = map(3,j);
            B_y = map(4,j);      
              
            if isincircle([A_x,A_y], radio_range, [B_x,B_y])
                I(i,j) = 1;
            end 
        end
    end
    
    %indirect interfaces
    children = find(CTT(1:nbr_of_routers,i)); %child routers of given router
    for j=1:length(children)
        node = children(j);
        
        A_x = map(3,node);
        A_y = map(4,node);
        
        for k=1:nbr_of_routers
            if xor((k==i),(~ismember(k,find(I(i,:)))))
                B_x = map(3,k);
                B_y = map(4,k);
                
                if isincircle([A_x,A_y], radio_range, [B_x,B_y])
                    I(i,k) = 1;
                end                
            end
        end        
    end
end

I = I + diag(ones(1,nbr_of_routers)*2); %add 2 on diagonal

%change nodes' number according mapping
for i=1:length(DF)
    for j=1:length(DF(i).sources)
        DF(i).sources(j) = find(map(1,:)==DF(i).sources(j));        
    end
    DF(i).sink = find(map(1,:)==DF(i).sink);
end









 

