function [startTime,q,feasible,noIntegerVar,ex_time] = ilpp_zig(n,m,per,qUB,p,A,W,dedication,filename,timelimit)
% A function solving an ILP model by different solvers selected in method.ilpSolver
% This function uses GLPK to generate ILP instance while using filename.mod written in MathProg
%
% PARAMETERS:
%  n            number of tasks
%  m            number of procesors
%  per          length of period
%  qUB          maximum number of execution periods
%  p            processing time
%  A            adjacency matrix constraining ofset start times
%  W            adjacency matrix constraining real start times               
%  dedication   dedication to given processors%  filename
%  timelimit    [sec] limit solution time
%
% Copyright Sucha Hanzalek {suchap,hanzalek}@fel.cvut.cz
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 

%global verbose     
global method
verbose=1; 
ex_time=0;

fullpath = strcat(pwd,'\'); %get current directory

filenameDataOut = [filename '.dat']; %data
filenameSolutionIn = [filename '.txt'];
filenameMod = [fullpath filename '.mod']; %model
filenameLPModel = [filename '.lp']; %problem in CPLEX LP format
filenameResult = [filename '.res']; %output file
%filenameCPLEXRes = [filename '.cpx'];
filenameCPLEXCommands = 'cplexcommands.txt';
posinf = '1e100';
neginf = '-1e100';


%Erase previous results
if(exist(filenameSolutionIn,'file'))
    delete(filenameSolutionIn);
end

if(exist(filenameResult,'file'))
    delete(filenameResult);
end


ilppglpkwriteprob_zig(filenameDataOut,n,m,per,qUB,p,A,W,dedication,posinf,neginf)
%call glpsol.exe with MathProg *.mod and *.dat to generate *.lp AND call chosen solver

switch(method.ilpSolver)
    case 'GLPKS'
        %Generate ILP model
        if(timelimit==0)
            cmd = [fullpath 'glpsol -m ' filenameMod ' -d ' filenameDataOut ' -o ' filenameResult ' --wcpxlp ' filenameLPModel ' > ' filenameSolutionIn];
        else
            cmd = [fullpath 'glpsol -m --tmlim ' num2str(timelimit) ' ' filenameMod ' -d ' filenameDataOut ' -o ' filenameResult ' --wcpxlp ' filenameLPModel ' > ' filenameSolutionIn];
        end

        tic;  %to measure time
        %Call GLPK
        [status,result] = system(cmd);
        ex_time=toc; 
        disp(sprintf('solving time = %d [sec]',ex_time));
        
        %Show error log if ILP terminated incorrectly
        if(~exist(filenameLPModel,'file'))
            disp(result)
            type(filenameSolutionIn);
            error('zig_tree:ilpp1','An error occured while processing ILP optimization.'); 
        end

        %Show constructe ILP equations
        if(verbose==2)
            type(filenameSolutionIn);
        end
        
        %read data from *.txt and *.res 
        [xmin,q,solutionStatus,noIntegerVar] = ilppglpkreadres_zig(filenameSolutionIn,filenameResult,filenameLPModel);
       
%         if(isa(procAssign,'double'))
%             conflictsBetweenTasks = procAssign;
%             procAssign = {};
%             %Clean 'procAssign'
%             for(i=1:length(dedication))
%                 for(j=1:length(dedication{i}))
%                     procAssign{i}(j) = 0;
%                 end
%             end
%             
%             for(j=1:length(cap))
%                 %Greedy coloring algorithm
%                 y = conflictsBetweenTasks;
%                 coloring = zeros(1,n);
%                 [foo,indices] = sort(xmin);
%                 for(i=indices)
%                     if(ismember(j,dedication{i}))
%                         if(cap(j)==Inf)
%                             %Ignore processors with unlimited capacity
%                             procAssign{i}(find(dedication{i}==j)) = 1;
%                             continue;
%                         end
%                         c = y(i,:) .* coloring;
%                         freeColors = setdiff([1:n],c);
%                         coloring(i) = freeColors(1);
%                         procAssign{i}(find(dedication{i}==j)) = coloring(i);
%                     end
%                 end
%             end
%        end
        
    case 'SCIPS'
        %Generate ILP model
        cmd = [fullpath 'glpsol -m ' filenameMod ' -d ' filenameDataOut ' -o ' filenameResult ' --wcpxlp ' filenameLPModel ' --check'];
        [status,result] = system(cmd);
        cmd = [fullpath 'scip -f ' filenameLPModel ' -q -l ' filenameSolutionIn];
        [status,result] = system(cmd);
        
        %Not finished yet.
        
    case 'CPLEX'
        %Generate ILP model
        cmd = [fullpath 'glpsol -m ' filenameMod ' -d ' filenameDataOut ' -o ' filenameResult ' --wcpxlp ' filenameLPModel ' --check'];
        
        [status,result] = system(cmd);
       
        %Show error log if GLPK (building ILP model) terminated incorrectly
        if(~exist(filenameLPModel,'file'))
            type(filenameSolutionIn);
            error(':ilpp_cdmuo','An error occur while processing ILP optimization.');
        end
        
        %Create a CPLEX script
        fid = fopen(filenameCPLEXCommands,'w');
        if(fid == -1)
            error(':ilpp',['Can not create file ''' filenameCPLEXCommands '''.']);
        end
        fprintf(fid,'read  %s\n',filenameLPModel);
        if(timelimit ~= 0)
            fprintf(fid,'set timelimit %d\n',timelimit);
        end
        fprintf(fid,'optimize\n');
        %fprintf(fid,'write %s\n',filenameSolutionIn);
        fprintf(fid,'display solution variables -\n');
        fprintf(fid,'quit\n');
        fclose(fid);

        %Call CPLEX
        cmd = ['cplex < ' filenameCPLEXCommands ' > ' filenameSolutionIn];
        tic;  %to measure time
        [status,result] = system(cmd);
        ex_time=toc; 
        disp(sprintf('solving time = %d [sec]',ex_time));

%PJ        [status,result] = system(cmd);
        
        %Show constructe ILP equations
        if(verbose==2)
            type(filenameSolutionIn);
        end


        [xmin,q,solutionStatus,noIntegerVar] = ilppcplexreadres_zig(filenameSolutionIn,filenameLPModel,length(p));

%         if(isa(procAssign,'double'))
%             conflictsBetweenTasks = procAssign;
%             procAssign = {};
%             %Clean 'procAssign'
%             for(i=1:length(dedication))
%                 for(j=1:length(dedication{i}))
%                     procAssign{i}(j) = 0;
%                 end
%             end
%             
%             for(j=1:length(cap))
%                 %Greedy coloring algorithm
%                 y = conflictsBetweenTasks;
%                 coloring = zeros(1,n);
%                 [foo,indices] = sort(xmin);
%                 for(i=indices)
%                     if(ismember(j,dedication{i}))
%                         if(cap(j)==Inf)
%                             %Ignore processors with unlimited capacity
%                             procAssign{i}(find(dedication{i}==j)) = 1;
%                             continue;
%                         end
%                         c = y(i,:) .* coloring;
%                         freeColors = setdiff([1:n],c);
%                         coloring(i) = freeColors(1);
%                         procAssign{i}(find(dedication{i}==j)) = coloring(i);
%                     end
%                 end
%             end
%         end
        
        
    otherwise
        error(':ilpp',['Solver ''' method.ilpSolver ''' is not supported.'])
end

%Count number of integer variables.
% fid = fopen(filenameLPModel,'r');
% if fid==-1
%     error(':ilpp1','An error occured while processing ILP optimization.');
% end
%
% startCount = 0;
% counterOfIntegers = 0;
% while 1
%     tline = fgetl(fid);
%     if ~ischar(tline), break, end
%     if strcmp(tline,'Generals'), startCount = 1; continue; end
%     if strcmp(tline,''), startCount = 0; end
%     if startCount==1, counterOfIntegers = counterOfIntegers +1; end
% end
% fclose(fid);
% 
% counterOfIntegers


%Check results
 if(~isempty(xmin))
     startTime=round(xmin(1:n));                %pridal Hanz, jelikoz CPLEX obcas asi spatne reprezentuje cele cislo
     feasible=solutionStatus;
 else
     disp('ILPP: There is not a feasible solution.');
     startTime=[];
     feasible=0;
 end;
