function [CTT,CD] = topologygen(Nrouter_max,Nend_node_max,no_routers)
%TOPOLOGYGEN random generation of cluster-tree topology
%
% PARAMETERS:
%   Nrouter_max     maximum number of child routers   
%   Nend_node_max   maximum number of child end_nodes
%   no_routers    total number of routers
%  
%   CTT     adjacency matrix - description of cluster-tree topology CTT
%   (logical topology) - parent-child relation
%   CD       collision domains
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%parameters
draw = 0; %[0,1]
x_max = 600; %size of area
y_max = 600;
tx_range = 25; %transmission range radius
cs_range = 40; %carrier-sense range radius
%%%%%%%%%%%%%%%

CTT = zeros(no_routers+no_routers*Nend_node_max); %parent-child relations [routers,end nodes]

%set of nodes' coordinates {[x,y]} [routers,end nodes]
nodes_coos = {[x_max/2,y_max/2]}; %root

if draw
    figure;
    hold on;
    plot(x_max/2,y_max/2,'ks'); %plot root    
    text(x_max/2+1,y_max/2+1,'1');
end

id_ch = 1; %cluster-head id
j = 0;
id_router = 1; %index of generated router(id of root = 1)

%Nrouter = Nrouter_max;
Nrouter = ceil((Nrouter_max-1)*rand)+1;

%%%%%%%%%%%%%%%%%
% cluster-tree topology
%%%%%%%%%%%%%%%%%
%generate no_routers routers - each router has Nrouter child routers
while id_router<no_routers
    %cluster head coordinates [x_ch,y_ch]
    x_ch = nodes_coos{id_ch}(1);
    y_ch = nodes_coos{id_ch}(2);   
    
    %integer random number from interval +-[0.5*tx_range, tx_range]
    x_dif = random_nbr(0.5*tx_range,tx_range);
    
    %[x,y] a router inside the transmission range of cluster head [x_ch,y_ch]
    x = x_ch+x_dif;
    y_bound = floor(sqrt(tx_range^2-abs(x_dif)^2)); %Pythagorean theorem 
    
    %integer random number from interval +-[0.5*y_bound, y_bound]
    y_dif = random_nbr(0.5*y_bound,y_bound); 
    y = y_ch+y_dif;
    
    nodes_coos = [nodes_coos, [x,y]]; %add router to the set
    id_router = id_router+1; %id of generated router
    
    CTT(id_router,id_ch) = 1; %parent-child relation [x,y]->[x_ch,y_ch]
    
    if draw
        plot(x,y,'go');
        text(x+1,y+1,int2str(id_router));
        plot(x_ch,y_ch,'go');
        plot([x_ch,x],[y_ch,y],'r');        
    end
    
    j=j+1; 
    if j>=Nrouter
        id_ch=id_ch+1;
        j=0;        
        Nrouter = floor((Nrouter_max+1)*rand); %generate random number of child routers
    end
end

id_end = no_routers; %index of generated end_node

%generate end nodes for each router - each router has Nend_node_max end nodes
for i=1:no_routers
    %router coordinates [x,y]
    x = nodes_coos{i}(1);
    y = nodes_coos{i}(2); 
    
    for z=1:Nend_node_max
        %integer random number from interval +-[1, tx_range/2]
        x_dif = random_nbr(1,tx_range/2);
        
        %[x_end,y_end] end_node coordinates
        x_end = x+x_dif;
        y_bound = floor(sqrt(tx_range^2-abs(x_dif)^2)); %Pythagorean theorem 
        
        %integer random number from interval +-[1, y_bound/2]
        y_dif = random_nbr(1,y_bound/2);
        y_end = y+y_dif;
        
        nodes_coos = [nodes_coos,[x_end,y_end]]; %add end node to the set of nodes
        id_end = id_end+1;  %id of generated end node
        
        CTT(id_end,i) = 1;  %parent-child relation [x_end,y_end]->[x,y]
        
        if draw
            plot(x_end,y_end,'bx');
            plot([x_end,x],[y_end,y],'b'); 
            text(x_end,y_end,int2str(id_end));
        end
    end
end

%%%%%%%%%%%%%%%%%
% collision domains
%%%%%%%%%%%%%%%%%

CD = diag(ones(1,no_routers));

for i=1:no_routers
    routers=i; %add current router
    children1 = find(CTT(1:no_routers,i)); %1-hop child routers of current router i
    end_nodes = find(CTT(no_routers+1:end,i))+no_routers; %child end nodes of current router i
    
    if i>1 %not root
        parent1 = find(CTT(i,1:no_routers)); %1-hop parent
        CD(i,parent1)=1; %1-hop parent collides
        routers=[routers,parent1]; %add 1-hop parent from list of routers
        parent2 = find(CTT(parent1,1:no_routers)); %2-hop parent
        if parent2
            CD(i,parent2)=1; %2-hop parent collides
            routers=[routers,parent2];%add 2-hop parent from list of routers
        end
    end
    
    if ~isempty(children1)
        CD(i,children1)=1; %1-hop child routers collide
        routers=[routers,children1']; %add 1-hop child routers from list of routers
        
        %2-hop child routers
        for j=1:length(children1)
            children2 = find(CTT(1:no_routers,children1(j))); %2-hop child routers
            CD(i,children2)=1; %2-hop child routers collide
            routers=[routers,children2'];  %add 2-hop child routers from list of routers
        end
    end    
    rem_routers = 1:no_routers;
    rem_routers(routers)=[]; %remove already used routers
    collision_area=[i,children1',end_nodes'];
    
    %remaining routers
    for j=1:length(rem_routers)
        isInside = 0;
        
        %coordinates of router j
        x_router = nodes_coos{rem_routers(j)}(1);
        y_router = nodes_coos{rem_routers(j)}(2); 
        
        %chil end nodes of router j
        end_nodes2 = find(CTT(no_routers+1:end,rem_routers(j)))+no_routers;
        
        %child routers of router j
        child_routers = find(CTT(1:no_routers,rem_routers(j)));
 
        %are router j or its end nodes and its child routers inside collision area?
        for z=1:length(collision_area)
            x = nodes_coos{collision_area(z)}(1);
            y = nodes_coos{collision_area(z)}(2);
            
            %router j
            if isincircle([x,y],cs_range,[x_router,y_router])
                isInside = 1;
                break;
            end
            
            %chil end nodes of router j
            for zz=1:length(end_nodes2)
                x_end2 = nodes_coos{end_nodes2(zz)}(1);
                y_end2 = nodes_coos{end_nodes2(zz)}(2);
                
                if isincircle([x,y],cs_range,[x_end2,y_end2])
                    isInside = 1;
                    break;
                end
            end
            
            %child routers of router j
            for zz=1:length(child_routers)
                x_child_router = nodes_coos{child_routers(zz)}(1);
                y_child_router = nodes_coos{child_routers(zz)}(2);
                
                if isincircle([x,y],cs_range,[x_child_router,y_child_router])
                    isInside = 1;
                    break;
                end
                
            end
            
            if isInside
                break;
            end   
        end
        
        if isInside
           CD(i,rem_routers(j))=1; %router collide
        end
    end    
end
