function [n,no_nodes,ex_time,feasible,status] = zigbee_tree(varargin)
%function [clusters_config,CTT,CD,DF,ctf] = zigbee_tree(varargin)
% ZIGBEE_TREE finds TDMA schedule for a ZigBee cluster-tree network  
%
% PARAMETERS:
%   g   object graph (output from graphedit tool)
%   ---------
%   CTT     adjacency matrix - description of cluster-tree topology CTT
%   (logical topology) - parent-child relation
%   CD      collision domains
%   DF      data flows
%   no_routers total number of routers
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global method
method.ilpSolver = 'GLPKS';      %'GLPKS' or 'CPLEX'

switch nargin
    case 0, 
        DF = dataflows;  %load data flow parameters
        [CTT,no_routers,CD] = topology;   %load topology parameters                       
    case 1, %from graphedit
        DF = dataflows;  %load data flow parameters
        [CTT,CD,map,DF,no_routers] = graph2inputs(varargin{1},DF); %load topology parameters
    case 3, %defined topology and data flows
        CTT = varargin{1};
        CD = varargin{2};
        DF = varargin{3};
        no_routers = length(CD);
    case 4,
        load conf CTT CD DF no_routers;
    otherwise,
        %generate random topology
        Nrouter = 3; %max number of child routers for each router
        Nend_node = 3; %max number of child end-node  for each router
        no_routers = 20; %total number of routers in WSN
        no_flows = 4; %number of data flows
        no_sources = 6; %number of sources for each flow

        [CTT,CD]=topologygen(Nrouter,Nend_node,no_routers); %Nrouter, Nend-node,#routers
        DF=dataflowgen(CTT,no_flows,no_sources); %no_flows,no_sources   
        
        save conf CTT CD DF no_routers;
end

parameters; %load IEEE 802.15.4/ZigBee parameters

%parent-child relation
no_nodes = length(CTT); %total number of nodes
PCHR = zeros(1,no_nodes); %parent-child relation. PCHRi is parent of node i.
for i=1:no_nodes
    parent = find(CTT(i,:)); %parent of node i
    if parent~=0 %node is not root
       PCHR(i)=parent;
    end
end

[ctf,clusters_proctime,interferences,BO_min,BO_max,SOs,links_req,max_deadline,status] = fill_ctf(CTT,DF,CD,no_routers); %fill the structure of cluster-tree flows

ex_time = 0;
feasible = 0;
n = 0;
    
disp(sprintf('status = %d',status));
if status>0 %PJ
    no_nodes = 0;
    return;
end

%get tasks and dedication to resources from the tree topology and destination and source addresses 
[p,A,W,dedication,proc_labels,task_labels,active_clusters,flow_links] = zig2tasks(ctf,clusters_proctime,interferences);

n = length(task_labels);  %number of tasks
m = length(proc_labels);	%number of processors
tlim = 600;    %deadline for GLPK solver [sec]
filename = 'zigbee';    %name of temporary files of GLPK solver 

BO = BO_min;

disp(sprintf('number of tasks = %d',n));
disp(sprintf('number of nodes = %d',no_nodes));

if BO_max < BO   
    disp(sprintf('BO_max (%d) < BO (%d)', BO_max, BO));
    return;
end

%BO=BO_max;
%iterative loop
while BO<=BO_max
    disp(sprintf('BO_max = %d, BO = %d', BO_max, BO));
    
    per = 16*2^BO;	%[#time slots] length of period
    %qUB = ceil(max_deadline/per);	%maximum number of execution periods
    qUB = 25;

    %solve the problem
    [startTime_new,q_new,feasible_new,noIntegerVar_new,ex_time] = ilpp_zig(n,m,per,qUB,p,A,W,dedication,filename,tlim);
    if feasible_new
        %update variables
        startTime = startTime_new;
        q = q_new;
        feasible = feasible_new;
        noIntegerVar = noIntegerVar_new;
        
        BO = BO + 1;
    else
        per = 16*2^(BO-1);
        break;
    end
end

%disp(sprintf('feasible = %d',feasible)); %PJ

if feasible
    disp('---------------------------------------');
    %configuration parameters of each cluster's head [id, startTime, SO, GTS]
    for i=1:length(active_clusters) 
        clusters_config(i).id = active_clusters(i);
        parent = PCHR(clusters_config(i).id);
        if parent==0 %root
            clusters_config(i).startTime = 0;
        else
            if startTime(i+1)<startTime(parent+1)
                startTime_TS = startTime(i+1)+per - startTime(parent+1); %dummy task (+1) [#time slots]
                clusters_config(i).startTime = startTime_TS*aBaseSlotDuration; %[symbols] rounded to back-off boundary
            else
                startTime_TS = startTime(i+1) - startTime(parent+1); %dummy task (+1) [#time slots]
                clusters_config(i).startTime = startTime_TS*aBaseSlotDuration; %[symbols] rounded to back-off boundary
            end
        end
        clusters_config(i).SO = SOs(clusters_config(i).id); %Superframe Order
        clusters_config(i).BO = BO - 1; %Beacon Order
        links = links_req(:,find(links_req(1,:)==clusters_config(i).id));
        
        slot = clusters_proctime(1,clusters_config(i).id)/2^clusters_config(i).SO; %first transmit slot
        
        GTS_T = [];     
        transmit = find(links(4,:)==0);
        for j=1:length(transmit) %transmit [IN] (to cluster-head)
            GTS_T(j).addr = links(2,transmit(j)); %associated device address
            GTS_T(j).length = links(5,transmit(j)); %length of GTS [# time slots]
            GTS_T(j).direction = 0;
            GTS_T(j).startSlot = slot;
            slot = slot + GTS_T(j).length;
        end
        
        GTS_R = [];
        receive = find(links(4,:)==1);
        for j=1:length(receive) %receive [OUT] (from cluster-head)
            GTS_R(j).addr = links(2,receive(j)); %associated device address
            GTS_R(j).length = links(5,receive(j)); %length of GTS [# time slots]
            GTS_R(j).direction = 1;
            GTS_R(j).startSlot = slot;
            slot = slot + GTS_R(j).length;
        end
        
        clusters_config(i).GTS = [GTS_T,GTS_R];
   
        %print
        disp(sprintf('cluster ID: %d',clusters_config(i).id));
        disp(sprintf('\t StartTime = %d [symbols] = %f [ms]',clusters_config(i).startTime,clusters_config(i).startTime*0.016));
        disp(sprintf('\t SO = %d, BO = %d',clusters_config(i).SO, clusters_config(i).BO));
        disp(sprintf('\t GTS parameters'));
        for j=1:size(clusters_config(i).GTS,2)
            disp(sprintf('\t\t GTS device: %d',clusters_config(i).GTS(j).addr));
            disp(sprintf('\t\t GTS length: %d',clusters_config(i).GTS(j).length));
            if clusters_config(i).GTS(j).direction
                disp(sprintf('\t\t GTS direction: %d (receive)',clusters_config(i).GTS(j).direction));
            else
                disp(sprintf('\t\t GTS direction: %d (transmit)',clusters_config(i).GTS(j).direction));
            end
            disp(sprintf('\t\t GTS starting slot: %d',clusters_config(i).GTS(j).startSlot));
            disp(sprintf('\t\t------'));
        end
    end
    
    disp('---------------------------------------');

%PJ
    %plot Gantt diagram
    figure;
    zigplot(p,dedication,task_labels,proc_labels,(1+length(active_clusters)),startTime,q,feasible);
    
    %data flows' deadline verification 
    for i=1:length(ctf)
        links_idx = find(flow_links(1,:)==i); %[id,src,dest]' of data flows
        sources = find(ctf(i).d_rel~=Inf); %data flow sources
        for j=1:length(sources)
            father = ctf(i).father(sources(j));
            while ctf(i).father(father)~=0
                father = ctf(i).father(father);
            end 

            source_idx = links_idx(find(flow_links(2,links_idx)==sources(j)));
            sink_idx = links_idx(find(flow_links(2,links_idx)==father));
            source_q = q(source_idx);
            source_startTime = startTime(source_idx);
            sink_q = q(sink_idx);
            sink_startTime = startTime(sink_idx);

            if ctf(i).rld(sources(j)) %rld = 1 (local) |OUT|
                source_startTime = source_startTime + clusters_proctime(1,sources(j))+clusters_proctime(2,sources(j)); %+CAP+IN
            else %rld = 0 (remote) |IN|OUT|
               source_startTime = source_startTime + clusters_proctime(1,sources(j)); %+CAP
            end

            if ctf(i).rld(father) %rld = 1 (local) |IN|
                sink_startTime = sink_startTime + clusters_proctime(1,father)+clusters_proctime(2,father); %+CAP+IN
            else %rld = 0 (remote) |IN|OUT|
                sink_startTime = sink_startTime + clusters_proctime(1,father)+clusters_proctime(2,father)+clusters_proctime(3,father); %+CAP+IN+OUT
            end

            e2e_delay = (sink_startTime + sink_q*per) - (source_startTime + source_q*per); %[#time slots]
            e2e_delay_sec = e2e_delay * (aBaseSuperframeDuration/16); %[sec]
            deadline = ctf(i).d_rel(sources(j)); %[#time slots]
            deadline_sec = deadline * (aBaseSuperframeDuration/16); %[sec]
            if deadline>=e2e_delay
                disp(sprintf('Flow %d, \t %d -> %d,\t E2E delay = %d [%f sec] \t (deadline = %d [%f sec])',i,sources(j),father,e2e_delay,e2e_delay_sec,deadline,deadline_sec));
            else
                error('Flow %d, \t %d -> %d,\t E2E delay = %d \t (deadline = %d [%f sec])',i,sources(j),father,e2e_delay,deadline,deadline_sec);
            end
        end    
    end
end
