function [ctf,cluster_proctime,interferences,BO_min,BO_max,SOs,links_req,max_deadline,status] = fill_ctf(CTT,DF,I,no_routers)
% FILL_CTF (Cluster-tree Flows) parameters of data flows (communication tasks)
%
% Parameters:
%   CTT     description of cluster-tree topology CTT (logical topology) -
%   parent-child relation. Adjacency matrix (directed graph): CTTij=1 if
%   node j is parent of node i. CTTij=0 otherwise. indexes: [routers,end_nodes] 
%
%   DF      data flows
%           DF.sources          set of sources (router or end-node) of the flow                           
%           DF.sink             a sink (router or end-node) of the flow
%           DF.deadline         [sec] deadline to message delivery from any
%           source to the sink
%           DF.sample_period    [sec] sample period of sensing data
%           DF.sammple_size     [bits] sample size of sensing data
%           DF.ack              [true/false] retransmission (packet with
%           acknowledgment)?
%   I       interference matrix; Iij = 1 if a direct or indirect
%   interference is between clusters (routers) i and j (i.e. clusters i and
%   j cannot be active at the same time))
%
%   no_routers  total number of routers (i.e. clusters) in WSN 
%
%   ctf                 structure of cluster-tree flows
%   cluster_proctime    [CAP,IN,OUT] the length of each cluster in time
%   slots
%   interferences       interferences among routers - a set of clusters
%   (routers) which cannot be active at the same time.      
%   BO_min          minimum Beacon Order (BO) - period
%   BO_max          maximum Beacon Order = minimum sample time
%   SOs             value of Superframe Order for each cluster
%   links_req       the GTS requirements of each cluster [cluster head, children, required bandwidth, direction, number of GTSs]
%   max_deadline    maximum deadline from each data flows
%   status          status
%                   1: Number of required GTSs is higher than 7
%                   2: The bandwith requirements are too high
%                   3: No flow for scheduling
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global aMinCAPLength aBaseSuperframeDuration CAP_TS;

%init
ctf = [];
cluster_proctime = [];
interferences = [];
BO_min = 0;
BO_max = 0;
SOs = [];
links_req = [];
max_deadline = 0;
status = 0;

nbr_of_nodes = length(CTT); %total number of nodes (routers + end-nodes) in WSN
DATA = zeros(size(CTT)); %[bits] required bandwidth at each link for all flows

%interferences among routers - a set of clusters (routers) which cannot be active at
%the same time. If the first cluster is active, the others in the set cannot be ative at the same time.  
%(e.g. {[1,2,3],[2,1,4,5],[3,1,6,7],[4,2],[5,2],[6,3],[7,3]})
for i=1:no_routers
    interferences{1,i}=[i, find(I(i,:)==1)];    
end

%find a set of clusters which can be active at the same time
%(parallel or non-interference clusters)
parallel_cl = {}; %set of parallel clusters (non-interference clusters)
clusters_rem = ones(1,no_routers); %remaining clusters marked with 1
II = I;
i = 1;
while i
    clusters_rem(i) = 0; %already used cluster
    II(:,i) = 2; %remove column
    ni_clusters = [i]; %non-interference clusters with given cluster
    fc = find(II(i,:)==0); %feasible clusters which have no interface with ni_clusters
    while fc
        ni_clusters = [ni_clusters, fc(1)];
        clusters_rem(fc(1)) = 0;
        II(:, fc(1)) = 2;
        fc = intersect(fc, find(II(fc(1),:)==0));
    end
    i = find(clusters_rem,1);
    parallel_cl = [parallel_cl, ni_clusters];
end

deadline_check = [];
idx = 1; %ctf index

CTT_links = CTT+CTT'; %data links (undirected)

CTT_floyd = CTT_links;
CTT_floyd(find(CTT_floyd==0)) = Inf;
CTT_floyd(1:length(CTT_floyd)+1:length(CTT_floyd)^2) = 0; %required format for Floyd alg.

[U,P] = floyd(CTT_floyd); %the shortest paths in the network
for i=1:length(DF) %for each data flow   
    %initialization
    father = ones(1, nbr_of_nodes)*-Inf; %next cluster of cluster i on the shortest path to the sink
    father_change = ones(1, nbr_of_nodes);
    rld = ones(1,no_routers)*Inf; %remote-local deadlines
    r = zeros(1,no_routers); %release date (availability of sensory data)
    d_abs = Inf; %total deadline of a given flow(from the start of a period till data delivery to the sink)
    d_rel = ones(1,no_routers)*Inf; %relative deadlines at each router along the shortest path (from the start of the data transmission till data delivery to the sink)

    DATA_cur = zeros(size(CTT)); %[bits] total size of data at each link for current data flow    
    
    %fill deadline for each source
    if length(DF(i).deadline)==1
        DF(i).deadline = ones(1,length(DF(i).sources))*DF(i).deadline;
    end
    
    for j=1:length(DF(i).sources) %for each source
        source = DF(i).sources(j);
        sink = DF(i).sink; 
        
        parent_src = find(CTT(source,:));
        if isempty(parent_src) %root
            parent_src = 0;
        end
        
        parent_sink = find(CTT(sink,:));
        if isempty(parent_sink) %root
            parent_sink = 0;
        end
        
        if CTT_links(source,sink)==1 %direct link between source and sink
            %no scheduling required - check deadline only 
            deadline_check = [deadline_check [source, sink, floor(DF(i).deadline(j)/(aBaseSuperframeDuration/16))]'];
            DATA_cur(source,sink) = DATA_cur(source,sink) + bandwidth(DF(i).sample_size,DF(i).ack); %required bandwidth        
        elseif parent_src==parent_sink %same parent for source and sink
            %no scheduling required - check deadline only 
            deadline_check = [deadline_check [source, sink, floor(DF(i).deadline(j)/(aBaseSuperframeDuration/16))]'];
            DATA_cur(source,parent_src) = DATA_cur(source,parent_src) + bandwidth(DF(i).sample_size,DF(i).ack); %required bandwidth
            DATA_cur(parent_sink,sink) = DATA_cur(parent_sink,sink) + bandwidth(DF(i).sample_size,DF(i).ack); %required bandwidth
        else
            father(sink) = 0; 
            sink_pred = P(source,sink);  %predecessor of the sink on the shortest path
            while source~=sink
                father(sink_pred) = sink;
                DATA_cur(sink_pred,sink) = DATA_cur(sink_pred,sink) + bandwidth(DF(i).sample_size,DF(i).ack); %required bandwidth
                sink = sink_pred;
                sink_pred = P(source,sink);
            end
            
            succ = father(source); %successor of source on the path
            if (source>no_routers)||(parent_src==succ) %source is end-node or child of its successor (is not active)
                deadline = floor(DF(i).deadline(j)/(aBaseSuperframeDuration/16)); %[time slot]
                max_deadline = max(max_deadline, deadline); %find maximal deadline
                d_rel(succ) = min(deadline, d_rel(succ));                
%                 if (CTT_links(source,DF(i).sink)~=1)&&(isempty(find(father==source)))%no direct link from source to sink                    
%                     father_change(source) = -Inf;                    
%                 end
                rld(succ) = 0; %remote source |IN|OUT|
            else %source is parent of its successor (is active)
                deadline = floor(DF(i).deadline(j)/(aBaseSuperframeDuration/16)); %[time slot]
                max_deadline = max(max_deadline, deadline); %find maximal deadline
                d_rel(source) = min(deadline, d_rel(source));
                rld(source) = 1; %local source |OUT|            
            end 
        end
    end
    
    for j=1:length(DF(i).sources) %for each source
        source = DF(i).sources(j);
        
        if (CTT_links(source,DF(i).sink)~=1)&&(isempty(find(father==source))) %no direct link from source to sink and no source as an intermediate node
            father(source) = -Inf;
        end       
    end
    
%    father = father.*father_change;
    
    parent_sink = find(CTT(DF(i).sink,:));
    if isempty(parent_sink) %root
       parent_sink = 0;
    end

    if ~isempty(find(father>0)) %flows for scheduling       
        pred = find(father==DF(i).sink);
        if (length(pred)>1) %more than one predecessor of the sink
            if DF(i).sink == 1 %sink is root
                rld(DF(i).sink) = 1; %local sink |IN|
            else %two sinks
                father(parent_sink) = 0; 
                rld(DF(i).sink) = 1;    %local 1.sink |IN|
                rld(find(CTT(DF(i).sink,:))) = 0;   %remote 2.sink |IN|OUT| 
            end
        elseif (DF(i).sink>no_routers)||(parent_sink==pred) %sink is end-node or child of its predecessor
            father(pred) = 0; % predecessor (CH) will be sink
            father(DF(i).sink) = -Inf;
            rld(pred) = 0; %remote sink |IN|OUT| 
        else
            rld(DF(i).sink) = 1; %local sink |IN|
        end              
       
        ctf(idx).father = father(1:no_routers);
        ctf(idx).rld = rld;
        ctf(idx).r = r;
        ctf(idx).d_abs = d_abs;
        ctf(idx).d_rel = d_rel;
        ctf(idx).task_names = [idx*ones(1,length(ctf(idx).father));[1:length(ctf(idx).father)];ctf(idx).father];
        idx = idx +1;
    else %no flow for scheduling
        %error('No flow for scheduling!');
        status = 3; %PJ
    end
    
    %DATA = DATA + bandwidth(DATA_cur, DF(i).ack); %[bits] calculate required bandwidth for each link   
    DATA = DATA +DATA_cur;
    sample_periods(i) = DF(i).sample_period; 
end   
    
%the GTS requirements of each cluster
% links_req = [cluster head, children, required bandwidth, direction]'
%routers' requirements
for i=1:no_routers
    children = find(CTT(1:no_routers,i)); %child routers of given router
    for j=1:length(children)
        if DATA(children(j),i) %transmit GTS (from child router to cluster-head)
            links_req = [links_req, [i, children(j), DATA(children(j),i), 0]']; 
        end
        if DATA(i,children(j)) %receive GTS (from cluster-head to child router)
            links_req = [links_req, [i, children(j), DATA(i,children(j)), 1]'];
        end        
    end 
end
%end-nodes' requirements
for i=1:length(DF)
    for j=1:length(DF(i).sources)
        source = DF(i).sources(j);
        if source > no_routers %source is end-node
            ch = find(DATA(source,:)~=0); %cluster head
            links_req = [links_req, [ch,source,DATA(source,ch), 0]']; %from end-node to cluster-head        
        end
    end
    if DF(i).sink > no_routers %sink is end-node
        ch = find(DATA(:,DF(i).sink)~=0); %cluster head 
        links_req = [links_req, [ch,DF(i).sink,DATA(ch,DF(i).sink), 1]']; %from cluster-head to end-node
    end
end

%processing time
SOs = ones(1,no_routers)*-1; %the length of each clusters (required SO values), -1: not used
GTS_req = zeros(1,size(links_req,2)); %number of GTS required for each link
for i=1:no_routers
   gts_nbr = find(links_req(1,:)==i); %number of required GTSs
   
   if length(gts_nbr)>7
       status = 1;  %PJ
       return;
       %error('Number of required GTSs is higher than 7!');
   end
   
   f = 1;
   SO = 0;   
   while (SO<=14) && f
       TS_req = 0; %number of required TSs in GTS with a given SO
       TS_time = (aBaseSuperframeDuration*2^SO)/16; %duration of a time slot for a given SO (2.4GHz@250kbps)
       TS_cap = TS_time*250000; %[bits] data capacity of a time slot for a given SO (2.4GHz@250kbps)
       for j=1:size(gts_nbr,2)
           GTS_req(gts_nbr(j)) = ceil(links_req(3,gts_nbr(j))/TS_cap);
           TS_req = TS_req + GTS_req(gts_nbr(j)); %number of required TSs with a given SO    
       end
   
       if TS_req > 16-max(CAP_TS, ceil(aMinCAPLength/TS_time))
           SO = SO + 1;
       else 
           f = 0;
       end
   end
   
   if f
       status = 2; %PJ
       return;
       %error('The bandwith requirements are too high!');
    else
        if TS_req~=0
            SOs(i) = SO;
        end
    end
end

%[cluster head, children, required bandwidth, direction, number of GTSs]
links_req = [links_req; GTS_req]; %add number of GTS required for each link

cluster_proctime = zeros(3, no_routers); %[CAP, IN, OUT];
for i=1:no_routers
    links = find(links_req(1,:)==i);
    in = 0;
    out = 0;
    for j=1:length(links)
        if links_req(4,links(j))==0 %0 = transmit [IN]
            in = in + links_req(5,links(j))*2^SOs(i); %[#time slot] for SO=0
        else %1 = receive [OUT]
            out = out + links_req(5,links(j))*2^SOs(i); %[#time slot] for SO=0
        end
    end
    if ~isempty(links)
        cap = 16*2^SOs(i) - (in + out);
        cluster_proctime(:,i)=[cap,in,out]';
    end
end

BI_max = min(sample_periods);   %[sec] theoretical maximum of Beacon Interval (BI)
BO_max = floor(log2(BI_max/aBaseSuperframeDuration));
BI_max = aBaseSuperframeDuration*2^BO_max;  %[sec] real maximum of BI

BI_min = 0;
for i=1:length(parallel_cl)
    SO_max = -1; %maximum SO from given parallel clusters
    for j=1:length(parallel_cl{i})
        SO_max = max(SO_max, SOs(parallel_cl{i}(j)));             
    end
    
    if SO_max > -1
        BI_min = BI_min + aBaseSuperframeDuration*2^SO_max; %minimum BI 
    end
end

BO_min = ceil(log2(BI_min/aBaseSuperframeDuration));


%check the local deadlines
for i=1:size(deadline_check,2)
    source = deadline_check(1,i);
    sink = deadline_check(2,i);
    deadline = deadline_check(3,i);
    
    if CTT(source,sink)==1  %direct link between source and sink, sink is parent
        if deadline<cluster_proctime(2,sink) %IN
            error(sprintf('Deadline from %d to %d is not feasible!',source,sink));
        else
            max_deadline = max(max_deadline, deadline); %find maximal deadline
        end
    elseif CTT(sink,source)==1 %direct link between sink and source, source is parent 
        if deadline<cluster_proctime(3,source) %OUT
            error(sprintf('Deadline from %d to %d is not feasible!',source,sink));
        else
            max_deadline = max(max_deadline, deadline); %find maximal deadline
        end
    else %same parent for source and sink
        parent = find(CTT(source,:));
        if deadline < (cluster_proctime(2,parent)+cluster_proctime(3,parent)) %IN + OUT
            error(sprintf('Deadline from %d to %d is not feasible!',source,sink));
        else
            max_deadline = max(max_deadline, deadline); %find maximal deadline
        end        
    end
end



